/*
 * 
 * Przykad sposobw pobierania danych
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;

using DevExpress.Xpo;
using DevExpress.Data.Filtering;

namespace QueryingDatastore1
{
    public class Customer : XPObject
    {
        public Customer() { }
        public Customer(Session session) : base(session) { }
        public Customer(string Name) { this.Name = Name; }
        public string Name;
        [Association("CustomerOrders")]
        public XPCollection<Order> Orders { get { return GetCollection<Order>("Orders"); } }
    }
    public class Order : XPObject
    {
        public Order() { }
        public Order(Session session) : base(session) { }
        public string Article;
        public Order(string Article) { this.Article = Article; }
        [Association("CustomerOrders")]
        public Customer CustomerID;
    }
    class Program
    {
        static void Main(string[] args)
        {
            XpoDefault.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];
            XpoDefault.Session.ClearDatabase();

            Customer c1 = new Customer("Jan Kowalski");
            Customer c2 = new Customer("Marian Malinowski");

            Order o1 = new Order("Zeszyt w kratk");
            Order o2 = new Order("Banany p tony");
            Order o3 = new Order("Szczotka do kibla");
            Order o4 = new Order("Drzwi wejciowe");
            Order o5 = new Order("Film Casino Royale");
            Order o6 = new Order("Serek wiejski");

            c1.Orders.Add(o1);
            c1.Orders.Add(o2);
            c2.Orders.Add(o3);
            c2.Orders.Add(o4);

            c1.Save();
            c2.Save();

            Console.WriteLine( "---------- XPCollection ----------" );

            XPCollection<Customer> xp = new XPCollection<Customer>(XpoDefault.Session);

            foreach (Customer c in xp)
            {
                Console.WriteLine(c.Name + ":");
                foreach (Order o in c.Orders)
                {
                    Console.WriteLine("    " + o.Article);
                }
            }

            Console.WriteLine( "---------- XPCursor ----------" ); // Robimy ledzenie w SQL Profiler

            XPCursor xpc = new XPCursor(XpoDefault.Session, typeof(Order));
            xpc.PageSize = 2;
            foreach (Order o in xpc)
            {
                Console.WriteLine(o.Article);
            }

            Console.WriteLine( "---------- XPView ----------" );

            XPView Letters = new XPView( XpoDefault.Session, typeof( Order ) );
            Letters.AddProperty("Litera", "Substring(Article, 1, 1)", true, true, DevExpress.Xpo.SortDirection.Ascending);

            for ( int i = 0; i < Letters.Count; i++ )
            {
                Console.WriteLine( (string)( Letters[i]["Litera"] ) );
            }

            Console.WriteLine( "---------- By ID ----------" );

            Order order = XpoDefault.Session.GetObjectByKey<Order>(1);
            Console.WriteLine(order.CustomerID.Name);

            Console.ReadLine();
        }
    }
}
