/*
 * 
 * Przykad uycia wzorca unit-of-work
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;

using DevExpress.Xpo;

namespace UnitOfWorkExample
{
    public class Person : XPObject
    {
        public Person() { }
        public Person(Session session) : base(session) { }
        private string firstName;
        public string FirstName
        {
            get { return firstName; }
            set
            {
                string oldValue = FirstName;
                if (oldValue == value) return;
                firstName = value;
                OnChanged("FirstName", oldValue, value);
            }
        }
        private string lastName;
        public string LastName
        {
            get { return lastName; }
            set
            {
                string oldValue = LastName;
                if (oldValue == value) return;
                lastName = value;
                OnChanged("LastName", oldValue, value);
            }
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            XpoDefault.Session.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];
            XpoDefault.Session.ClearDatabase();

            // Prosty zapis obiektw
            using (UnitOfWork uow = new UnitOfWork())
            {
                uow.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];
                Person p = new Person( uow );
                p.FirstName = "Jan01";
                p.LastName = "Kowalski01";
                Person p2 = new Person(uow);
                p2.FirstName = "Jan02";
                p2.LastName = "Kowalski02";
                uow.CommitChanges();
            }

            // Aktualizacja obiektw (zwracamy uwag, e aktualizowane s tylko te zmienione)
            using (UnitOfWork uow = new UnitOfWork())
            {
                uow.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];
                XPCollection<Person> xpc = new XPCollection<Person>(uow);
                Person p1 = xpc[0];
                p1.FirstName = "Jan01X";
                p1.LastName = "Kowalski01X";
                Person p2 = xpc[1];
                p2.FirstName = "Jan02";
                p2.LastName = "Kowalski02";
                uow.CommitChanges();
            }

            using (UnitOfWork uow = new UnitOfWork())
            {
                uow.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];
                //uow.LockingOption = LockingOption.Optimistic;
                XPCollection persons1 = new XPCollection(uow, typeof(Person));

                Person p1 = (Person)persons1[0];
                p1.FirstName = "Jan 2.0";                

                //using (NestedUnitOfWork uowNested = uow.BeginNestedUnitOfWork())
                using (UnitOfWork uowNested = new UnitOfWork())
                {
                    uowNested.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];
                    //uowNested.LockingOption = LockingOption.None;
                    XPCollection persons2 = new XPCollection(uowNested, typeof(Person));
                    Person p2 = (Person)persons2[1];
                    p1.Delete();
                    p1.FirstName = "Jan 4.0";
                    p2.LastName = "Kowalski 4.0";
                    uowNested.CommitChanges();
                }
                uow.CommitChanges();
            }
        }
    }
}
